import 'location.dart';
import 'syntax_error.dart';
import 'token.dart';
import 'token_type.dart';

class BaseParser {
  int _index = -1;
  final String filename;
  Location location;
  final List<Token> tokens;

  BaseParser(this.filename, this.tokens) {
    location = new Location(filename, 1, 0);
  }

  bool eof() => _index >= tokens.length;

  SyntaxError expected(String msg) {
    return new SyntaxError(current != null ? '${current.location} $msg' : msg);
  }

  SyntaxError expectedType(TokenType type) {
    return expected('Expected $type, ${current?.type ?? "nothing"} found.');
  }

  Token backtrack([int n]) => read((n ?? 1) * -1);

  Token get current => eof() || _index == -1 ? null : tokens[_index];

  bool next(TokenType type) {
    if (_index >= tokens.length - 1) {
      return false;
    }

    // print('Current is ${current?.type}. Is next $type?');

    if (peek()?.type == type) {
      // print('Yes.');
      read();
      return true;
    }

    // print('No.');
    return false;
  }

  Token peek([int n]) => tokens[_index + (n ?? 1)];

  Token peekBack([int n]) => peek((n ?? -1) * -1);

  Token read([int n]) {
    final tok = tokens[_index += (n ?? 1)];
    // print('Read token: "${tok.text}" -> ${tok.type}');
    return tok;
  }
}
